#pragma once

#include <functional>

#pragma warning( push )
#pragma warning( disable : 4290 )

template < class T, T Invalid = NULL >
class ThrowOnErrorTraits : public std::unary_function< bool, T const & >
{
public:
	bool operator() (T const & t) const
	{
		return t != Invalid;
	}
};

template < class T, class Traits = ThrowOnErrorTraits< T > >
class ThrowOnError
{
public:
	ThrowOnError(void) throw()
		: m_t(),
		m_inval()
	{
	}

	ThrowOnError(T const & t) throw(T)
		: m_t( t ),
		m_inval()
	{
		Check();
	}

	ThrowOnError(ThrowOnError< T, Traits > const & rhs) throw(T)
		: m_t( static_cast< T > (rhs) ),
		m_inval()
	{
		Check();
	}

	ThrowOnError< T, Traits > & operator = (ThrowOnError< T, Traits > const & rhs) throw(T)
	{
		if (&rhs != this)
		{
			m_t = static_cast< T > (rhs);
			Check();
		}
		return *this;
	}

	ThrowOnError< T, Traits > & operator = (T const & t) throw(T)
	{
		m_t = t;
		Check();
		return *this;
	}

	operator T() const throw()
	{
		return m_t;
	}

private:
	void Check() throw(T)
	{
		if (!m_inval( m_t ))
			throw m_t;
	}

private:
	T m_t;
	const Traits m_inval;
};

class InvalidHRESULT : public std::unary_function< bool, HRESULT >
{
public:
	bool operator() (HRESULT hr) const
	{
		return SUCCEEDED(hr);
	}
};

typedef ThrowOnError< HRESULT, InvalidHRESULT > HResult;

#pragma warning( pop )

